from .canvas import Canvas
from http.server import BaseHTTPRequestHandler, HTTPServer
import webbrowser


def svg_server(canvas: Canvas, port: int = 8000):
    class MyServer(BaseHTTPRequestHandler):
        def do_GET(self):
            self.send_response(200)
            self.send_header("Content-type", "text/html")
            self.end_headers()
            self.wfile.write(bytes("<html><head><title>Python SVG</title></head>", "utf-8"))
            self.wfile.write(bytes("<body>", "utf-8"))
            self.wfile.write(bytes(canvas.svg_content(), "utf-8"))
            self.wfile.write(bytes("</body></html>", "utf-8"))

    webServer = HTTPServer(("localhost", port), MyServer)
    print(f"Server started http://localhost:{port}")

    webbrowser.open_new_tab(f"http://localhost:{port}")

    try:
        webServer.serve_forever()
    except KeyboardInterrupt:
        pass

    webServer.server_close()
    print("Server stopped.")




from typing import List


class Canvas:
    def __init__(self, width: int, height: int) -> None:
        self.width: int = width
        self.height: int = height
        self.shapes: List = []

    def svg_content(self) -> str:
        content = f'<svg width="{self.width}" height="{self.height}">'
        for shape in self.shapes:
            content += f"\n {shape.svg_content()}"
        content += "\n </svg>"
        return content

    def add_shape(self, shape):
        self.shapes.append(shape)



from shapes import Rectangle
from link import Link
from canvas import Canvas
from server import svg_server

canvas = Canvas(500, 500)
link = Link("Something", "https://google.com", canvas)
rect = Rectangle(100, 200, link)
svg_server(canvas)




import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()


setup(
    name="python_svg",
    version="0.0.9",
    description="Create Dynamic SVGs using Python3",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/KrishnaKanth1729/PySVG",
    author="KrishnaKanth1729",
    author_email="rkrishnakanth1729@gmail.com",
    packages=find_packages(),
    keywords=['python', 'svg'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


from .link import Link
from .canvas import Canvas
from typing import Union, List, Optional
from .style import Style


class Shape:
    def __init__(self, parent):
        self.parent: Union[Canvas, Link] = parent
        self.area: int = 0

    def svg_content(self) -> str:
        pass


class Rectangle(Shape):
    def __init__(self, width: int, height: int, parent: Union[Canvas, Link], x: int = 0, y: int = 0,
                 style: Optional[Style] = Style({"fill": "black"}),
                 rx: Optional[int] = 0, ry: Optional[int] = 0):
        """
        Rectangle class

        :param width
        :param height: int
        :param parent
        :param x:
        :param y:
        :param style:
        :param rx:
        :param ry:
        """
        super().__init__(parent)
        parent.add_shape(self)
        self.width = width
        self.height = height
        self.area = self.width * self.height
        self.x = x
        self.y = y
        self.style = style
        self.rx = rx
        self.ry = ry

    def svg_content(self) -> str:
        content = f'<rect width="{self.width}" height="{self.height}" x={self.x} y={self.y} style={self.style.svg()} rx={self.rx} ry={self.ry} />'
        return content


class Circle(Shape):
    def __init__(self, cx: int, cy: int, r: int, style: Style, parent: Union[Canvas, Link], stroke: str = "",
                 stroke_width: int = None, fill: str = None):
        if not isinstance(parent, Canvas) or isinstance(parent, Link):
            raise ValueError(f"Expected a class of Canvas or Link found type {type(parent)}")

        super().__init__(parent)
        self.parent.add_shape(self)
        self.cx = cx
        self.cy = cy
        self.r = r
        self.style = style
        self.stroke = stroke
        self.stroke_width = stroke_width
        self.fill = fill

    def svg_content(self):
        return f'<circle cx="{self.cx}" cy="{self.cy}" r="{self.r}" stroke="{self.stroke}" stroke-width="{self.stroke_width}" fill="{self.fill}" style={self.style.svg()} />'


class Point(Shape):
    def __init__(self, x: int, y: int, canvas: Canvas):
        super().__init__(canvas)
        self.x = x
        self.y = y
        self.canvas = canvas
        self.check_outline()

    def check_outline(self):
        if self.x > self.canvas.width:
            raise Warning(f"X {self.x} value of point {self} lies outside canvas of width {self.canvas.width}")
        elif self.y > self.canvas.height:
            raise Warning(f"x {self.x} value of point {self} lies outside canvas of width {self.canvas.width}")

    def __str__(self):
        return f"{self.x},{self.y}"


class Polygon(Shape):
    def __init__(self, points: List[Point], parent: Union[Canvas, Link], style: Style = Style({'fill': 'black'})):
        if not isinstance(parent, Canvas) or isinstance(parent, Link):
            raise ValueError(f"Expected a class of Canvas or Link found type {type(parent)}")

        super().__init__(parent)
        parent.add_shape(self)
        self.points = points
        self.style = style

    def svg_content(self):
        return f'<polygon points="{" ".join([str(point) for point in self.points])}" style={self.style.svg()} />'


class Ellipse(Shape):
    def __init__(self, cx: int, cy: int, rx: int, ry: int, parent: Union[Canvas, Link], style: Style = Style({'fill': 'black'})):
        if not isinstance(parent, Canvas) or isinstance(parent, Link):
            raise ValueError(f"Expected a class of Canvas or Link found type {type(parent)}")

        super().__init__(parent)
        parent.add_shape(self)
        self.cx = cx
        self.cy = cy
        self.rx = rx
        self.ry = ry
        self.canvas = parent
        self.style = style

    def svg_content(self) -> str:
        return f'<ellipse cx="{self.cx}" cy="{self.cy}" rx="{self.rx}" ry="{self.ry}" style="{self.style.svg()}" />'


class Line(Shape):
    def __init__(self, x1: int, y1: int, x2: int, y2: int, parent: Union[Canvas, Link], style: Style = Style({'stroke': 'black'})):
        if not isinstance(parent, Canvas) or isinstance(parent, Link):
            raise ValueError(f"Expected a class of Canvas or Link found type {type(parent)}")

        super().__init__(parent)
        parent.add_shape(self)

        self.x1 = x1
        self.x2 = x2
        self.y1 = y1
        self.y2 = y2
        self.canvas = parent
        self.style = style

    def svg_content(self) -> str:
        return f'<line x1="{self.x1}" y1="{self.y1}" x2="{self.x2}" y2="{self.y2}" style="{self.style.svg()}" />'


class Text(Shape):
    def __init__(self, x: int, y: int, parent: Union[Canvas, Link], fill: str = "black", transform: Optional[str] = ""):
        if not isinstance(parent, Canvas) or isinstance(parent, Link):
            raise ValueError(f"Expected a class of Canvas or Link found type {type(parent)}")

        super().__init__(parent)
        parent.add_shape(self)
        self.x = x
        self.y = y
        self.canvas = parent
        self.fill = fill
        self.transform = transform

    def svg_content(self) -> str:
        return f'<text x="{self.x}" y="{self.y}" fill="{self.fill}" transform="{self.transform}"></text>'



from typing import Optional, List
from .shapes import Shape
from svg.canvas import Canvas


class Link(Shape):
    def __init__(self, text_arg: str, href: Optional[str], canvas: Canvas):
        """
        Link class for HTML <a> tags

        :param text_arg:
        :param href:
        """
        super().__init__(canvas)
        self.parent.add_shape(self)
        self.text_arg = text_arg
        self.href = href
        self.shapes: List[Shape] = []

    def svg_content(self) -> str:
        content = f'<a href="{self.href}">'
        for shape in self.shapes:
            content += shape.svg_content()
        content += '</a>'
        return content

    def add_shape(self, shape) -> None:
        self.shapes.append(shape)

